/*---------------------------------------------------------------------------*/
/* GIVE.C -- This program allows the unprivileged user to copy an arbitrary  */
/*   file to a user who has given him permission to do so.                   */
/*---------------------------------------------------------------------------*/

#include <jpidef.h>
#include <ssdef.h>
#include <descrip.h>
#include <uaidef.h>
#include <stdio.h>
#include <prvdef.h>

#include "give.h"
#include "give-cli.h"
#include "give-dest.h"
#include "give-auth.h"
#include "give-rms.h"
#include "give-log.h"
#include "give-prv.h"

/*------*/
/* Constant declarations                                                     */

#define K_USERNAME_PARM "USERNAME"
#define K_FILENAME_PARM "FILENAME"
#define K_SETUP_QUAL "SETUP"
#define K_DEBUG_QUAL "DEBUG"

/*------*/
/* Globals                                                                   */

boolean qual_debug;

/*---------------------------------------------------------------------------*/
/* Print_Usage -- prints a usage message.                                    */

void Print_Usage( void )
{
   puts( "GIVE -- user-to-user file transfer utility\n" );
   puts( "Usage:  GIVE filename username  (to transfer a file)" );
   puts( "        GIVE/SETUP              (to set up GIVE for receiving)" );
   puts( "        GIVE/NEW                (to check for new received files)" );
   puts( "        GIVE/CHECK              (to verify authorization file)\n" );
}

/*---------------------------------------------------------------------------*/
/* main -- the usual                                                         */

int main( int argc, char **argv, char **envp ) {

   ulong ss_stat;
   
   char parm_username[K_USERNAME_LEN+1];
   char parm_filename[K_FILENAME_LEN+1];
   boolean qual_setup;
   char destination_directory[K_FILENAME_LEN+1];
   char login_directory[K_FILENAME_LEN+1];
   char destination_filename[K_FILENAME_LEN+1];
   char destination[K_FILENAME_LEN+1];
   char logfile_name[K_FILENAME_LEN+1];
   $DESCRIPTOR( destination_d, destination );
   ulong destination_diskquota;
   ulong source_file_size;

/*------*/
/* Disable installed privileges                                              */

   Priv_Disable( PRV$M_SYSPRV | PRV$M_WORLD | PRV$M_OPER | PRV$M_SYSLCK );
   

/*------*/
/* Get the command line arguments                                            */

   Get_Presence( K_SETUP_QUAL, &qual_setup );
   Get_Presence( K_DEBUG_QUAL, &qual_debug );
   Get_Value( K_USERNAME_PARM, parm_username, sizeof( parm_username ) );
   Get_Value( K_FILENAME_PARM, parm_filename, sizeof( parm_filename ) );

   if( qual_setup )
      exit( SS$_ABORT );

   if( parm_filename[0] == '\0' ) {
      Print_Usage();
      exit( GIVE_NOFILENAME );
   }
   if( parm_username[0] == '\0' ) {
      Print_Usage();
      exit( GIVE_NOUSERNAME );
   }

/*------*/
/* Get destination information                                               */

   Priv_Enable( PRV$M_SYSPRV );
   ss_stat = Get_Destination_Directory( parm_username, login_directory );
   Priv_Disable( PRV$M_SYSPRV );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );
   strcpy( destination_directory, login_directory );

   if( qual_debug )
      printf( "Destination directory: %s\n", destination_directory );

   Priv_Enable( PRV$M_SYSPRV );
   ss_stat = Get_Destination_Diskquota( destination_directory, parm_username,
                                        &destination_diskquota );
   Priv_Disable( PRV$M_SYSPRV );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

   if( qual_debug )
      printf( "Destination diskquota: %d\n", destination_diskquota );

/*------*/
/* Get source file information                                               */

   ss_stat = Get_File_Size( parm_filename, &source_file_size );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

   if( qual_debug )
      printf( "Source file size: %d\n", source_file_size );

/*------*/
/* Get authorization information                                             */

   strcpy( destination_filename, parm_filename );
   Priv_Enable( PRV$M_SYSPRV );
   ss_stat = Check_Authorization( destination_directory, source_file_size,
                                  destination_diskquota,
                                  destination_filename );
   Priv_Disable( PRV$M_SYSPRV );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

   if( qual_debug )
      printf( "Destination filename: %s%s.\n", destination_directory,
              destination_filename );

/*------*/
/* Attempt to create the destination directory.                              */

   strcpy( destination, destination_directory );
   destination_d.dsc$w_length = strlen( destination );

   Priv_Enable( PRV$M_SYSPRV );
   ss_stat = lib$create_dir( &destination_d, 0, 0, 0, 0, 0 );
   Priv_Disable( PRV$M_SYSPRV );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

/*------*/
/* Copy the file to the destination directory.                               */

   strcat( destination, destination_filename );

   Priv_Enable( PRV$M_SYSPRV );
   ss_stat = Copy_File( parm_filename, destination );
   Priv_Disable( PRV$M_SYSPRV );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

/*------*/
/* Append an informative message to the log file.                            */

   Priv_Enable( PRV$M_SYSPRV | PRV$M_SYSLCK );
   ss_stat = Log_Give( login_directory, parm_username, destination );
   Priv_Disable( PRV$M_SYSPRV | PRV$M_SYSLCK );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

/*------*/
/* Send a broadcast message to the destination username.                     */

   Priv_Enable( PRV$M_WORLD | PRV$M_OPER );
   ss_stat = Log_Broadcast( parm_username, destination );
   Priv_Disable( PRV$M_WORLD | PRV$M_OPER );
   if( ss_failure( ss_stat ) )
      exit( ss_stat );

}
